@@ -1,5 +1,9 @@
+# Originally from https://github.com/openai/CLIP. MIT License, Copyright (c) 2021 OpenAI
+
 import os
+import io
 import urllib
+import shutil
 import warnings
 from typing import Union, List
 
@@ -33,7 +37,7 @@ _MODELS = {
 }
 
 
-def _download(url: str, root: str):
+def _download(url: str, root: str, with_resume: bool = True):
     os.makedirs(root, exist_ok=True)
     filename = os.path.basename(url)
 
@@ -67,20 +71,48 @@ def _download(url: str, root: str):
 
         task = progress.add_task('download', filename=url, start=False)
 
-        with urllib.request.urlopen(url) as source, open(
-            download_target, 'wb'
-        ) as output:
+        tmp_file_path = download_target + '.part'
+        resume_byte_pos = (
+            os.path.getsize(tmp_file_path) if os.path.exists(tmp_file_path) else 0
+        )
 
-            progress.update(task, total=int(source.info().get('Content-Length')))
+        total_bytes = -1
+        try:
+            # resolve the 403 error by passing a valid user-agent
+            req = urllib.request.Request(url, headers={'User-Agent': 'Mozilla/5.0'})
 
-            progress.start_task(task)
-            while True:
-                buffer = source.read(8192)
-                if not buffer:
-                    break
+            total_bytes = int(
+                urllib.request.urlopen(req).info().get('Content-Length', -1)
+            )
 
-                output.write(buffer)
-                progress.update(task, advance=len(buffer))
+            mode = 'ab' if (with_resume and resume_byte_pos) else 'wb'
+
+            with open(tmp_file_path, mode) as output:
+
+                progress.update(task, total=total_bytes)
+
+                progress.start_task(task)
+
+                if resume_byte_pos and with_resume:
+                    progress.update(task, advance=resume_byte_pos)
+                    req.headers['Range'] = f'bytes={resume_byte_pos}-'
+
+                with urllib.request.urlopen(req) as source:
+                    while True:
+                        buffer = source.read(8192)
+                        if not buffer:
+                            break
+
+                        output.write(buffer)
+                        progress.update(task, advance=len(buffer))
+        except Exception as ex:
+            raise ex
+        finally:
+            # rename the temp download file to the correct name if fully downloaded
+            if os.path.exists(tmp_file_path) and (
+                total_bytes == os.path.getsize(tmp_file_path)
+            ):
+                shutil.move(tmp_file_path, download_target)
 
     return download_target
 
@@ -89,9 +121,14 @@ def _convert_image_to_rgb(image):
     return image.convert('RGB')
 
 
-def _transform(n_px):
+def _blob2image(blob):
+    return Image.open(io.BytesIO(blob))
+
+
+def _transform_blob(n_px):
     return Compose(
         [
+            _blob2image,
             Resize(n_px, interpolation=BICUBIC),
             CenterCrop(n_px),
             _convert_image_to_rgb,
@@ -104,6 +141,20 @@ def _transform(n_px):
     )
 
 
+def _transform_ndarray(n_px):
+    return Compose(
+        [
+            ToTensor(),
+            Resize(n_px, interpolation=BICUBIC),
+            CenterCrop(n_px),
+            Normalize(
+                (0.48145466, 0.4578275, 0.40821073),
+                (0.26862954, 0.26130258, 0.27577711),
+            ),
+        ]
+    )
+
+
 def available_models() -> List[str]:
     '''Returns the names of available CLIP models'''
     return list(_MODELS.keys())
@@ -143,6 +194,7 @@ def load(
         model_path = _download(
             _S3_BUCKET + _MODELS[name],
             download_root or os.path.expanduser('~/.cache/clip'),
+            with_resume=True,
         )
     elif os.path.isfile(name):
         model_path = name
@@ -168,7 +220,11 @@ def load(
         model = build_model(state_dict or model.state_dict()).to(device)
         if str(device) == 'cpu':
             model.float()
-        return model, _transform(model.visual.input_resolution)
+        return (
+            model,
+            _transform_blob(model.visual.input_resolution),
+            _transform_ndarray(model.visual.input_resolution),
+        )
 
     # patch the device names
     device_holder = torch.jit.trace(
@@ -233,7 +289,11 @@ def load(
 
         model.float()
 
-    return model, _transform(model.input_resolution.item())
+    return (
+        model,
+        _transform_blob(model.input_resolution.item()),
+        _transform_ndarray(model.input_resolution.item()),
+    )
 
 
 def tokenize(
